function [status,msg] = impose(obj,str,tol,gap)
%IMPOSE Impose new canonical structure to the start node.
%   IMPOSE(SGOBJ,STR,Tol,Gap) imposes the canonical structure STR on the
%   matrix data in SGOBJ.
%   [Fail, MSG] = IMPOSE(SGOBJ,STR,Tol,Gap) also returns Fail == 0 
%   if success, otherwise Fail > 0 and MSG is the error message.  
%
%   See also SGCOM/ASSIMILATE.

% This file is part of the Matlab plugin for StratiGraph.
% This file is freely-available under the modified
% BSD license.


if ~hasdata(obj)
	status = 2;  
	msg = 'Missing data. Probably no numerical data is associated with the structure.';
	return;
end
  
msg = '';
status = 0; % Success status
try
	if isa(str,'mstruct')
		[S,P,E,eigv] = miguptri(obj.A,str,tol,gap,'zeros');
		obj.A = P*S*P';
	elseif isa(str,'pstruct')
		[S,T,P,Q,E,F,eigv] = piguptri(obj.A,obj.B,str,tol,gap,'zeros');
		obj.A = P*S*Q';
		obj.B = P*T*Q';
	elseif isa(str,'ssstruct')
		[S,T,P,E,F,eigv] = s2iguptri(oftype(str),obj.A,obj.B,str,tol, ...
		gap,'zeros');
		obj.A = P*S*P'; % P * [Bbar, Abar-zI] * [Ip 0; 0 P]' /  P * [Abar-zI; Cbar] * [P 0; 0 Iq]'
		obj.B = P*T;
	else
		error('Unsupported Structure %s.',class(str));
	end
	% Set imposed eigenvalues
	jb = str.get('fjblock','object');
	for k = 1:length(jb)
		jb{k}.set('eigenvalue',eigv(k));
	end
	obj.mlstruct = str;
catch e
	msg = e.message;
	status = 1; % Error status
end