function str = jstruct2mlstruct(jstrCont,varargin)
%JSTRUCT2MLSTRUCT Java structure object to Matlab structure object.
%   Mobj = JSTRUCT2MLSTRUCTC(Jobj) returns the Matlab canonical
%   structure object Mobj in the StratiGraph structure container Java
%   object Jobj.
%
%   Mobj = JSTRUCT2MLSTRUCTC(Jobj,Eigv) also provides the imposed
%   eigenvalues on a structure. Eigv is of type
%   se.umu.stratigraph.core.util.Eigenvalue.

% This file is part of the Matlab plugin for StratiGraph.
% This file is freely-available under the modified
% BSD license.

import se.umu.stratigraph.core.plugin.StructureContainer;
import se.umu.stratigraph.core.structure.*;
import se.umu.stratigraph.core.util.Eigenvalue;

jstr = jstrCont.getStructure();

% If the structure is empty do a quick return
if jstr.isEmpty()
  str = [];
  return
end

if nargin > 1 && ~isempty(varargin{1})
  eigvo = varargin{1};
else
  eigvo = jstr.getRegularIndices().getEigenvalues();
end
rilo = jstr.getRegularIndices().toArray();
leneig = length(eigvo);
eigv = zeros(leneig,1);
J = cell(leneig,1);

for k = 1:leneig
  blk = double(rilo(k).getSegreBlocks());
  if isempty(eigvo(k)) || eigvo(k).isNaN()
    eigv(k) = NaN;
    J{k}    = blk;
  elseif eigvo(k).isZero()
    eigv(k) = 0.0;
    J{k}    = blk;
  else
    eigv(k) = eigvo(k).real() + 1i*eigvo(k).imag();
    J{k}    = blk;
  end
end
  
cname = strsplit(class(jstr),'.');
switch cname{end}
 case 'PencilStructure'
  R = double(jstr.getIndices(BlockType.R).getSegreBlocks());
  L = double(jstr.getIndices(BlockType.L).getSegreBlocks());
  N = double(jstr.getIndices(BlockType.N).getSegreBlocks());
  str = pstruct(R,L,J,eigv,N);
 case 'MatrixStructure'
  str = mstruct(J,eigv);
 case 'StateSpaceStructure'
  R = double(jstr.getIndices(BlockType.R).getSegreBlocks());
  L = double(jstr.getIndices(BlockType.L).getSegreBlocks());
  N = double(jstr.getIndices(BlockType.N).getSegreBlocks());
  str = ssstruct(R,L,J,eigv,N);
 case 'MatrixPolyStructure'
  R = double(jstr.getIndices(BlockType.R).getSegreBlocks());
  L = double(jstr.getIndices(BlockType.L).getSegreBlocks());
  N = double(jstr.getIndices(BlockType.N).getSegreBlocks());
  d = jstrCont.getSizeParameters().getParameter('d').getValue();
  
  str = mpstruct(d,R,L,J,eigv,N);
 otherwise
  error('Not a recognized structure: %s',cname{end});
end
	