function [labels] = sglabels
%SGLABELS Return the labels of the opened StratiGraph windows.
%   Labels = SGLABELS returns a cell array with the labels of the opened
%   StratiGraph windows.
%
%   See also SGOPEN, SGCLOSE.

% This file is part of the Matlab plugin for StratiGraph.
% This file is freely-available under the modified
% BSD license.


narginchk(0,0);
nargoutchk(0,1);

% Fetch all open SG window labels
try
  jobj = se.umu.stratigraph.Main.windowIDs;
catch me
  % Try to find the jar-files in parent directory and add them to classpath
  jdir = [fileparts(which('sglabels')) '/..'];
  D = dir([jdir '/*.jar']);
  if isempty(D)
    me = MException('MCS:sgopen',['No jar files found. StratiGraph ' ...
                        'is probably not installed correct.']);
    throwAsCaller(me)
  end
  for k = 1:length(D)
    jarpath = which(D(k).name);
    if isempty(jarpath)
      em = MException('MCS:sgopen', ['Jar-file %s is not in the ' ...
                          'Matlab search path. Add MCS root ' ...
                          'folder to the Matlab search path.'], ...
                      D(k).name);
      throwAsCaller(me)
    end
    javaaddpath(jarpath)
  end
  try
    jobj = se.umu.stratigraph.Main.windowIDs;
  catch
    me = MException('MCS:sgopen', ['Is StratiGraph compiled for ' ...
                        'the correct Java version?\n Running ' ...
                        'Matlab %s uses JRE\n %s'], version, ...
                    version('-java'));
    throwAsCaller(me)
  end
end

labels = cell(size(jobj));
for i=1:length(jobj)
  labels{i} = char(jobj(i));
end

global MCS_PREF;

% Remove any closed windows from MCS_PREF.sgwin
if isfield(MCS_PREF,'sgwin') && ~isempty(MCS_PREF.sgwin)
  [nil,sgi,mcsi] = intersect(labels',{MCS_PREF.sgwin(:).label});
  MCS_PREF.sgwin = MCS_PREF.sgwin(sort(mcsi));
end
